using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using SourceGrid2;
using SourceGrid2.BehaviorModels;
using Cells = SourceGrid2.Cells.Real;

// TODO : Better in grid validation

namespace ConfigEdit
{
    public partial class ConfigEdit : UserControl
    {
        #region Privates
        // Member variables
        private Boolean IsDirty = false;

        // Grid settings
        private SourceGrid2.DataModels.IDataModel m_CellEditor_String;
		private SourceGrid2.VisualModels.Common m_VisualProperties;
        //private SourceGrid2.VisualModels.Header m_ModelRowHeader;
        #endregion

        #region Defines

        // Header Information
        private static string DTSCONFIGURATION = "DTSConfiguration";
        private static string HEADING = "DTSConfigurationHeading";
        private static string FILEINFO = "DTSConfigurationFileInfo";
        private static string FILEINFOXPATH = "/" + DTSCONFIGURATION + "/" + HEADING + "/" + FILEINFO;
        private static string GENERATEDBY = "GeneratedBy";
        private static string GENERATEDNAME = "GeneratedFromPackageName";
        private static string GENERATEDID = "GeneratedFromPackageID";
        private static string GENERATEDDATE = "GeneratedDate";
        private static string MODIFIEDDATE = "LastModifiedDate";
        private static string CONFIGDESC = "Description";
        private static string DEFAULTDESCRIPTION = "Integration Services Package Configuration file. This file was generated in the Package Configurations Editor.";
        
        // Configurations Information
        private static string CONFIGUREDTYPE = "ConfiguredType";
        private static string PROPERTY = "Property";
        private static string PATH = "Path";
        private static string TYPE = "ValueType";
        private static string CONFIGUREDVALUE = "ConfiguredValue";
        private static string CONFIGURATION = "Configuration";
        private static string PACKAGEPREFIX = "\\Package";

        #endregion
        
        #region Constructor
        // Constructor
        public ConfigEdit()
        {
            InitializeComponent();
            lErrorText.Visible = false;

            ConfigGrid.RowsCount = 1;
            ConfigGrid.ColumnsCount = 4;
            ConfigGrid.FixedRows = 1;
            ConfigGrid.FixedColumns = 1;
            ConfigGrid.Selection.SelectionMode = SourceGrid2.GridSelectionMode.Row;
            ConfigGrid.AutoStretchColumnsToFitWidth = true;
            ConfigGrid.Columns[0].AutoSizeMode = SourceGrid2.AutoSizeMode.None;
            ConfigGrid.Columns[0].Width = 30;

            // Initialize the Grid            
            Cells.Header l_00Header = new Cells.Header();
            ConfigGrid[0, 0] = l_00Header;

            m_CellEditor_String = SourceGrid2.Utility.CreateDataModel(typeof(string));
            m_CellEditor_String.EditableMode = SourceGrid2.EditableMode.Focus | SourceGrid2.EditableMode.AnyKey | SourceGrid2.EditableMode.SingleClick;
//            m_ModelRowHeader = (SourceGrid2.VisualModels.Header)SourceGrid2.VisualModels.Header.RowHeader.Clone(false);

            ConfigGrid[0, 1] = new Cells.ColumnHeader("Package Path");
            ConfigGrid[0, 2] = new Cells.ColumnHeader("Value Type");
            ConfigGrid[0, 3] = new Cells.ColumnHeader("Value");

        }
        #endregion

        #region GridClasses
        private class RowHeaderIndex : SourceGrid2.Cells.Real.RowHeader
        {
            public override string DisplayText
            {
                get
                {
                    return this.Row.ToString();
                }
            }
        }
        #endregion

        #region ConfigEditMethods

        // Clear the grid and the error text.
        public void Clear()
        {
            if( ConfigGrid.RowsCount > 1 )
                ConfigGrid.Rows.RemoveRange(1, ConfigGrid.RowsCount - 1);                
            ClearErrorText();
            IsDirty = false;
        }

        // Create a new row at the bottom of the grid
        private void AddARow()
        {
            ConfigGrid.Rows.Insert(ConfigGrid.RowsCount,
                new RowHeaderIndex(),
                new Cells.Cell(m_CellEditor_String.DefaultValue, m_CellEditor_String, m_VisualProperties),
                new Cells.Cell(m_CellEditor_String.DefaultValue, m_CellEditor_String, m_VisualProperties),
                new Cells.Cell(m_CellEditor_String.DefaultValue, m_CellEditor_String, m_VisualProperties)
                );
        }

        // Remove a row at the index
        private void RemoveARow(int index)
        {
            if( ConfigGrid.RowsCount > index )
                ConfigGrid.Rows.Remove(index);
        }        
                
        // Perform some validity checks on each configuration.        
        private void ValidateConfiguration(XmlNode node, int index)
        {
            // Validate the Package Path attribute
            if( node.Attributes[PATH] == null )
            {
                AddErrorText(Errors.PathAttributeNotFound + index.ToString());
            }
            else if( node.Attributes[PATH].InnerText.Length < 10 ||
                       node.Attributes[PATH].InnerText.Substring(0, 8) != PACKAGEPREFIX )
            {
                AddErrorText(Errors.PackagePrefixNotFound + index.ToString() );
            }

            // Validate the ValueType attribute
            if( node.Attributes[TYPE] == null )
            {
                AddErrorText(Errors.ValueTypeAttributeNotFound + index.ToString());
            }
            
            XmlNode value = node.FirstChild;
            if (value == null || value.Name != CONFIGUREDVALUE)
            {
                AddErrorText(Errors.ConfiguredValueNodeNotFound + index.ToString());                
            }            
            else if( value.InnerText.Length == 0 )
            {
                AddErrorText(Errors.ValueEmpty + index.ToString());
            }
        }

        // Loads a row in the grid for each configuration entry in the configuration file.
        private void LoadRows(XmlDocument doc)
        {        
            m_VisualProperties = new SourceGrid2.VisualModels.Common(false);

            XmlNodeList nlRows = doc.SelectNodes("//Configuration");
            ConfigGrid.RowsCount = nlRows.Count + 1;
            int RowsIndex = 1;

            ConfigGrid[0, 1] = new Cells.ColumnHeader("Package Path");
            ConfigGrid[0, 2] = new Cells.ColumnHeader("Value Type");
            ConfigGrid[0, 3] = new Cells.ColumnHeader("Value");

            foreach (XmlNode node in nlRows)
            {
                try
                {
                    ValidateConfiguration(node, RowsIndex);

                    // Add the row header on the left margin
                    ConfigGrid[RowsIndex, 0] = new RowHeaderIndex();
                    //ConfigGrid[RowsIndex, 0].VisualModel = m_ModelRowHeader;

                    // Get the path
                    if (node.Attributes[PATH] != null)
                    {
                        ConfigGrid[RowsIndex, 1] = new Cells.Cell(node.Attributes[PATH].InnerText);
                        ConfigGrid[RowsIndex, 1].DataModel = m_CellEditor_String;
                        ConfigGrid[RowsIndex, 1].VisualModel = m_VisualProperties;
                    }

                    // Get the ValueType
                    if (node.Attributes[TYPE] != null)
                    {
                        ConfigGrid[RowsIndex, 2] = new Cells.Cell(node.Attributes[TYPE].InnerText);
                        ConfigGrid[RowsIndex, 2].DataModel = m_CellEditor_String;
                        ConfigGrid[RowsIndex, 2].VisualModel = m_VisualProperties;
                    }

                    // Get the value node, show error if not there.
                    XmlNode value = node.FirstChild;
                    if (value != null)
                    {
                        ConfigGrid[RowsIndex, 3] = new Cells.Cell(value.InnerText);
                        ConfigGrid[RowsIndex, 3].DataModel = m_CellEditor_String;
                        ConfigGrid[RowsIndex, 3].VisualModel = m_VisualProperties;
                    }
                        
                     RowsIndex++;                    
                } // Catch any exceptions, give an error and attempt to continue loading.
                catch (Exception e)
                {
                    AddErrorText(e.Message);
                }
            }
        }

        // Save the configuration entries.
        private void SaveRows(XmlDocument doc, XmlNode parent)
        {
            int countRows = ConfigGrid.RowsCount;
            for (int i = 1; i < countRows; i++)
            {
                XmlElement node = doc.CreateElement(CONFIGURATION);

                XmlAttribute attr = doc.CreateAttribute(CONFIGUREDTYPE);
                attr.InnerText = PROPERTY;
                node.Attributes.Append(attr);

                attr = doc.CreateAttribute(PATH);                
                attr.InnerText = ConfigGrid[i,1].DisplayText;
                node.Attributes.Append(attr);

                attr = doc.CreateAttribute(TYPE);
                attr.InnerText = ConfigGrid[i, 2].DisplayText;
                node.Attributes.Append(attr);

                XmlElement value = doc.CreateElement(CONFIGUREDVALUE);
                value.InnerText = ConfigGrid[i, 3].DisplayText;
                node.AppendChild(value);

                // Only append if the configuration is valid.
                // Don't care about ValueType since runtime ignores it anyway.
                if (ConfigGrid[i, 1].DisplayText.Length > 0 &&
                    ConfigGrid[i, 3].DisplayText.Length > 0)
                {
                    parent.AppendChild(node);
                }
                else
                {
                    AddErrorText(Errors.ConfigurationNotSaved + i.ToString());
                }
            }
        }
        
        // Track if an edit has been made.              
        public Boolean Dirty
        {
            get
            {
                return IsDirty;
            }
            set
            {
                IsDirty = value;
            }
        }


        // Create a new configuration with no configuration entries.
        public void CreateNewXMLConfiguration()
        {
            // Get a fresh start on errors.
            ClearErrorText();

            try
            {

                lGeneratedBy.Text = System.Environment.MachineName;
                lPackageName.Text = "";
                lPackageID.Text = "";
                lDateCreated.Text = System.DateTime.Today.ToShortDateString();
                lDateLastModified.Text = System.DateTime.Today.ToShortDateString();
                tbDescription.Text = DEFAULTDESCRIPTION;
                Clear();
                AddARow();                
                IsDirty = true;
            }
            catch (Exception e)
            {
                // Set the error control value to the exception text.
                AddErrorText(e.Message);
            }
        }
        
        // Load an existing XML configuration files
        public void LoadXMLConfiguration( string FileName )
        {
            // Get a fresh start on errors.
            ClearErrorText();

            // Create the document to load.
            System.Xml.XmlDocument doc;
            doc = new XmlDocument();
            try
            {
                doc.Load(FileName);

                //
                // Get the header information if it's available.
                //

                // Create a temporary node to hold the configuration values
                XmlNode tempNode;
                string tempString = "";
                XmlAttribute tempAttr;

                // Get the header node
                tempNode = doc.SelectSingleNode(FILEINFOXPATH);
                if (tempNode != null)
                {

                    // Get the GeneratedBy attribute.
                    tempAttr = tempNode.Attributes[GENERATEDBY];
                    if (tempAttr != null)
                    {
                        tempString = tempAttr.InnerText;
                        if (tempString.Length > 0)
                            lGeneratedBy.Text = tempString;
                    }

                    // Get the GeneratedName attribute.
                    tempAttr = tempNode.Attributes[GENERATEDNAME];
                    if (tempAttr != null)
                    {
                        tempString = tempAttr.InnerText;
                        if (tempString.Length > 0)
                            lPackageName.Text = tempString;
                    }

                    // Get the GeneratedID attribute.
                    tempAttr = tempNode.Attributes[GENERATEDID];
                    if (tempAttr != null)
                    {
                        tempString = tempAttr.InnerText;
                        if (tempString.Length > 0)
                            lPackageID.Text = tempString;
                    }

                    // Get the GeneratedDate attribute.
                    tempAttr = tempNode.Attributes[GENERATEDDATE];
                    if (tempAttr != null)
                    {
                        tempString = tempAttr.InnerText;
                        if (tempString.Length > 0)
                            lDateCreated.Text = tempString;
                    }

                    // Get the ModifiedDate attribute.
                    tempAttr = tempNode.Attributes[MODIFIEDDATE];
                    if (tempAttr != null)
                    {
                        tempString = tempAttr.InnerText;
                        if (tempString.Length > 0)
                            lDateLastModified.Text = tempString;
                    }

                    // Get the ConfigurationDescription attribute.
                    tempAttr = tempNode.Attributes[CONFIGDESC];
                    if (tempAttr != null)
                    {
                        tempString = tempAttr.InnerText;
                        if (tempString.Length > 0)
                            tbDescription.Text = tempString;
                    }

                    LoadRows(doc);
                }
                IsDirty = false;
            }
            catch (Exception e)
            {            
                // Set the error control value to the exception text.
                AddErrorText(e.Message);
            }
        }
        public Boolean SaveXMLConfiguration(string FileName)
        {
            // Get a fresh start on errors.
            ClearErrorText();

            System.Xml.XmlDocument doc;
            doc = new XmlDocument();
            try
            {
                //
                // Create XML File
                //

                // XML Declaration
                XmlDeclaration xmlDec = doc.CreateXmlDeclaration("1.0", "", "");
                doc.AppendChild(xmlDec);

                // Create the root and append to document.
                XmlNode Root = doc.CreateNode("element", DTSCONFIGURATION, "");
                doc.AppendChild(Root);

                //
                // Create the header element.
                //
                XmlNode Heading = doc.CreateNode("element", HEADING, "");
                Root.AppendChild( Heading );

                //
                // Create the file info element.
                //
                XmlNode FileInfo = doc.CreateNode("element", FILEINFO, "");
                Heading.AppendChild(FileInfo);
                
                XmlAttribute tempAttr;

                // Get the values for each of the file information fields and add to the configuration.
                // Generated By
                tempAttr = doc.CreateAttribute(GENERATEDBY);                
                tempAttr.InnerText = lGeneratedBy.Text;                
                FileInfo.Attributes.Append( tempAttr );

                // Generated Name
                tempAttr = doc.CreateAttribute(GENERATEDNAME);                
                tempAttr.InnerText = lPackageName.Text;                
                FileInfo.Attributes.Append( tempAttr );
                
                // Generated ID
                tempAttr = doc.CreateAttribute(GENERATEDID);
                tempAttr.InnerText = lPackageID.Text;                
                FileInfo.Attributes.Append( tempAttr );
                
                // Generated Date
                tempAttr = doc.CreateAttribute(GENERATEDDATE);
                if (lDateCreated.Text.Length < 5)
                    lDateCreated.Text = System.DateTime.Today.ToShortDateString();                
                tempAttr.InnerText = lDateCreated.Text;
                FileInfo.Attributes.Append( tempAttr );

                // Modified Date, update to today since we're saving now
                tempAttr = doc.CreateAttribute(MODIFIEDDATE);                
                lDateLastModified.Text = System.DateTime.Today.ToShortDateString();
                tempAttr.InnerText = lDateLastModified.Text;
                FileInfo.Attributes.Append( tempAttr );
                
                // Configuration Description
                tempAttr = doc.CreateAttribute(CONFIGDESC);
                tempAttr.InnerText = tbDescription.Text;
                FileInfo.Attributes.Append( tempAttr );

                SaveRows(doc, Root);

                doc.Save(FileName);
                IsDirty = false;
            }
            catch (Exception e)
            {
                AddErrorText(e.Message);
                return false;
            }
            return true;
        }
        #endregion //ConfigurationSection

        #region HelperFunctions

        #region ErrorTextFunctions
        //
        // ErrorText methods 
        //
        // Clears and sets the error text window text
        protected void SetErrorText(string errorText)
        {
            lErrorText.Text = errorText;
            lErrorText.Visible = true;
        }

        // Append error to error text window
        protected void AddErrorText(string errorText)
        {
            if (lErrorText.Text.Length > 0)
                lErrorText.Text = lErrorText.Text + "\r";

            lErrorText.Text = lErrorText.Text + errorText;
            lErrorText.Visible = true;
        }
        
        // Hide and clear the error text        
        protected void ClearErrorText()
        {
            lErrorText.Text = "";
            lErrorText.Visible = false;
        }
        #endregion
        #endregion

        #region PrivateControlSection

        private void ConfigEdit_Load(object sender, EventArgs e)
        {

        }
        #endregion

        #region EventHandlers
        // Fired when the description edit box changes.
        private void tbDescription_TextChanged(object sender, EventArgs e)
        {
            IsDirty = true;
        }

        // Fired when the Add button is clicked.
        private void AddConfig_Click(object sender, EventArgs e)
        {
            AddARow();

            ConfigGrid.Rows[ConfigGrid.RowsCount - 1].Focus();

            IsDirty = true;
        }

        // Fired when the Remove button is clicked.
        private void RemoveConfig_Click(object sender, EventArgs e)
        {
            try
            {
                int i = 1;
                SourceGrid2.RowInfo[] Rows = ConfigGrid.Selection.SelectedRows;
                foreach (SourceGrid2.RowInfo r in Rows)
                {
                    i = r.Index;
                    RemoveARow(r.Index);
                }

                // Try to place the focus on the closest row
                if (ConfigGrid.RowsCount > i)
                    ConfigGrid.Rows[i].Focus();
                // Default to the second row if it exists. 
                // The first row is the zero based header.
                else if (ConfigGrid.RowsCount > 1)
                    ConfigGrid.Rows[1].Focus();

                IsDirty = true;
            }
            catch (Exception ex)
            {
                SetErrorText(ex.Message);
            }
        }
        #endregion
    }
}