using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Microsoft.DataTransformationServices.Controls;
using Microsoft.SqlServer.Dts.Design;
using Microsoft.SqlServer.Dts.Runtime;
using Microsoft.SqlServer.Dts.Runtime.Design;
using theTask = SSIS2K8.Samples.Tasks.CryptoTask;

namespace SSIS2K8.Samples.Tasks.CryptoTask
{
    #region SettingsView
   public partial class SettingsView : System.Windows.Forms.UserControl, IDTSTaskUIView
   {
      #region private members
      private const string NEW_CONN = "<New Connection...>";

      private System.Windows.Forms.PropertyGrid settingsPropertyGrid;
      private SettingsNode settingsNode = null;
      private CryptoTask theTask = null;

      #region Init and Dispose
      public SettingsView()
      {
         // This call is required by the Windows.Forms Form Designer.
         InitializeComponent();
      }
      #endregion Init and Dispose
      #endregion

      #region accessors
      public object SettingsNode
      {
         get
         {
            return settingsNode;
         }
      }
      #endregion

      #region IDTSTaskUIView
      /// <summary>
      /// Called when the view is about to be displayed for the first time
      /// </summary>
      /// <param name="treeHost">IDTSTaskUIHost that host the view</param>
      /// <param name="taskHost">the task that is edited by TaskUI</param>
      /// <param name="viewNode">the node in the tree associated with this view</param>
      public virtual void OnInitialize(IDTSTaskUIHost treeHost, System.Windows.Forms.TreeNode viewNode, object taskHost, object connections)
      {
         if (taskHost == null)
         {
            throw new ArgumentNullException("Attempting to initialize the CryptoTask UI with a null TaskHost");
         }

         if (!(((TaskHost)taskHost).InnerObject is CryptoTask))
         {
            throw new ArgumentException("Attempting to initialize the CryptoTask UI with a task that is not a CryptoTask.");
         }

         theTask = ((TaskHost)taskHost).InnerObject as CryptoTask;

         this.settingsNode = new SettingsNode(taskHost as TaskHost, connections );

         settingsPropertyGrid.SelectedObject = this.settingsNode;
      }

      /// <summary>
      /// is called when the OK button is pressed
      /// </summary>
      /// <param name="bViewIsValid">result of view validation</param>
      public virtual void OnValidate(ref bool bViewIsValid, ref string reason)
      {
      }

      /// <summary>
      /// called whenever the view is displaying
      /// </summary>
      public virtual void OnSelection()
      {
      }

      /// <summary>
      /// called every time the view is about to be left.
      /// </summary>
      /// <param name="bCanLeaveView">true if the view can be left</param>
      public virtual void OnLoseSelection(ref bool bCanLeaveView, ref string reason)
      {
      }

      /// <summary>
      /// called when OK button is pushed, but only if all OnValidate calls succeeded
      /// the user should set whatever properties need to be set to the task by this view
      /// </summary>
      /// <param name="taskHost">the edited task.</param>
      public virtual void OnCommit(object taskHost)
      {
         // Commit the changes from this view
         theTask.EncryptionOperation = settingsNode.EncryptionOption;
         theTask.SourceFileConnectionManager = settingsNode.SourceCMName;
         theTask.DestinationFileConnectionManager = settingsNode.DestCMName;
         theTask.OverwriteDestination = settingsNode.OverwriteDestination; 

      }
      #endregion IDTSTaskUIView

      #region Component Designer generated code
      /// <summary> 
      /// Required method for Designer support - do not modify 
      /// the contents of this method with the code editor.
      /// </summary>
      private void InitializeComponent()
      {
         this.settingsPropertyGrid = new System.Windows.Forms.PropertyGrid();
         this.SuspendLayout();
         // 
         // settingsPropertyGrid
         // 
         this.settingsPropertyGrid.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                     | System.Windows.Forms.AnchorStyles.Left)
                     | System.Windows.Forms.AnchorStyles.Right)));
         this.settingsPropertyGrid.Location = new System.Drawing.Point(3, 0);
         this.settingsPropertyGrid.Name = "settingsPropertyGrid";
         this.settingsPropertyGrid.PropertySort = System.Windows.Forms.PropertySort.Categorized;
         this.settingsPropertyGrid.Size = new System.Drawing.Size(387, 400);
         this.settingsPropertyGrid.TabIndex = 0;
         this.settingsPropertyGrid.ToolbarVisible = false;
         // 
         // SettingsView
         // 
         this.Controls.Add(this.settingsPropertyGrid);
         this.Name = "SettingsView";
         this.Size = new System.Drawing.Size(390, 400);
         this.ResumeLayout(false);
         this.settingsPropertyGrid.PropertyValueChanged += new System.Windows.Forms.PropertyValueChangedEventHandler(this.propertyGridSettings_PropertyValueChanged);
      }
      #endregion

      #region PropertyGrid Event handler
      private void propertyGridSettings_PropertyValueChanged(object s, System.Windows.Forms.PropertyValueChangedEventArgs e)
      {
         #region New File Connection for Source path
         if (e.ChangedItem.PropertyDescriptor.Name.CompareTo("SourceConnection") == 0)//Do not localise  
         {
            if (e.ChangedItem.Value.Equals(NEW_CONN))
            {
               ArrayList list = null;

               this.Cursor = Cursors.WaitCursor;
               if (!((settingsNode.SourceCMName == null) || (settingsNode.SourceCMName == "")))
                  settingsNode.SourceCMName = null;

               list = ((IDtsConnectionService)settingsNode.Connections).CreateConnection("FILE");
               this.Cursor = Cursors.Default;
               if ((list != null) && (list.Count > 0))
               {
                  ConnectionManager cMgr = (ConnectionManager)list[0];

                  settingsNode.SourceCMName = cMgr.Name;
               }
               else
               {
                  if (e.OldValue == null)
                  {
                     settingsNode.SourceCMName = null;
                  }
                  else
                  {
                     settingsNode.SourceCMName = (string)e.OldValue;
                  }
               }
            }
         }
         #endregion New File Connection

         #region New File Connection for destination path
         else if (e.ChangedItem.PropertyDescriptor.Name.CompareTo("DestinationConnection") == 0)//Do not localise  
         {
            if (e.ChangedItem.Value.Equals(NEW_CONN))
            {
               ArrayList list = null;

               this.Cursor = Cursors.WaitCursor;
               if (!((settingsNode.DestCMName == null) || (settingsNode.DestCMName == "")))
                  settingsNode.DestCMName = null;

               list = ((IDtsConnectionService)settingsNode.Connections).CreateConnection("FILE");
               this.Cursor = Cursors.Default;
               if ((list != null) && (list.Count > 0))
               {
                  ConnectionManager cMgr = (ConnectionManager)list[0];

                  settingsNode.DestCMName = cMgr.Name;
               }
               else
               {
                  if (e.OldValue == null)
                  {
                     settingsNode.DestCMName = null;
                  }
                  else
                  {
                     settingsNode.DestCMName = (string)e.OldValue;
                  }
               }
            }
         }
         #endregion New File Connection
      }
      #endregion Property Grid Event Handler

   }
   #endregion

	#region File Connections
	/// <summary>
	/// Returns FILE Connections available in the package
	/// </summary>
	internal class FileConnections : StringConverter
	{
      private const string NEW_CONN = "<New Connection...>";

		private object GetSpecializedObject(object contextInstance)
		{
			DTSLocalizableTypeDescriptor typeDescr = contextInstance as DTSLocalizableTypeDescriptor;

			if (typeDescr == null)
			{
				return contextInstance;
			}

			return typeDescr.SelectedObject;
		}

		public override StandardValuesCollection GetStandardValues(ITypeDescriptorContext context)
		{
			object retrievalObject = GetSpecializedObject(context.Instance) as object;

			return new StandardValuesCollection(getFileConnections(retrievalObject));
		}

		public override bool GetStandardValuesExclusive(ITypeDescriptorContext context)
		{
			return true;
		}

		public override bool GetStandardValuesSupported(ITypeDescriptorContext context)
		{
			return true;
		}

		/// <summary>
		/// Method to get the the file connections created or available in the package
		/// </summary>
		/// <param name="retrievalObject"></param>
		/// <returns>list of FILE connections</returns>
		private ArrayList getFileConnections(object retrievalObject)
		{
        SettingsNode node = (SettingsNode)retrievalObject;
         
        Connections runTimeCons = ((IDtsConnectionService)node.Connections).GetConnections();
         ArrayList list = new ArrayList();

			list.Add(NEW_CONN);
			foreach (ConnectionManager conn in runTimeCons)
			{
				if (conn.CreationName.CompareTo("FILE") == 0 || conn.CreationName.CompareTo("FLATFILE") == 0)
				{
					list.Add(conn.Name);
				}
			}

			if ((list != null) && (list.Count > 0))
				list.Sort();

			return list;
		}
	}
	#endregion Fileconnections

	#region Operation Types
	/// <summary>
	/// Returns File System Task operations
	/// </summary>
	internal class EncryptOperationTypes : StringConverter
	{
		private string[] _encryptoperationTypes = 
      {	
         theTask.CryptoTask.PERSIST_CONST_ENCRYPT,
         theTask.CryptoTask.PERSIST_CONST_DECRYPT
      };
			
		public override StandardValuesCollection GetStandardValues(ITypeDescriptorContext context) 
		{
			return new StandardValuesCollection (_encryptoperationTypes);
		}
		public override bool GetStandardValuesExclusive(ITypeDescriptorContext context) 
		{
			return true;
		}
		public override bool GetStandardValuesSupported(ITypeDescriptorContext context)
		{
			return true;
		}
	}
   #endregion Operation Types
    
    #region SettingsNode
   /// <summary>
   /// Settings Node class corresponds to the view that is displayed when
   /// the user clicks the Settings node.
   /// </summary>
   internal class SettingsNode
   {
      #region private members

      private CryptoTask.EncryptOp _EncryptOp = CryptoTask.EncryptOp.Encrypt;
      private string _sourceConnection = null;
      private string _destConnection = null;
		private bool _overwriteDestination = false;

      // Interface references
		private object _connections = null;
      
      // Not needed for this task, but leaving for reference.
      //internal IDtsVariableService _variableService = null; 

      // Task and Taskhost references
      private theTask.CryptoTask _task = null;
      private TaskHost _taskHost = null;

      #endregion private members

      #region Initialization
      public SettingsNode(TaskHost taskHost, object connections)
      {
         // Initialize members
         _taskHost = taskHost;
         _task = taskHost.InnerObject as SSIS2K8.Samples.Tasks.CryptoTask.CryptoTask;
         _connections = connections;
         _sourceConnection = _task.SourceFileConnectionManager;
         _destConnection = _task.DestinationFileConnectionManager;
         _overwriteDestination = _task.OverwriteDestination;
         _EncryptOp = _task.EncryptionOperation;
        
         // Not needed for this task, but leaving for reference.
         // _variableService = taskHost.Site.GetService(typeof(IDtsVariableService)) as IDtsVariableService;
      }
      #endregion

      #region Properties
      // Internal Properties
      // Connections reference
      [
      Browsable(false)
      ]
      internal object Connections
      {
         get { return _connections; }
      }

      // Source connection manager name
      [
      Browsable(false)
      ]
      internal string SourceCMName
      {
         get { return _sourceConnection; }
         set { _sourceConnection = value; }
      }

      // Destination connection manager name
      [
      Browsable(false)
      ]
      internal string DestCMName
      {
         get { return _destConnection; }
         set { _destConnection = value; }
      }

      // Public Properties
      [
      Category("Encryption Properties"),
      Description("Where you enter the write-only password."),
      PasswordPropertyText(true)
      ]
      public string Password
      {
         get { return "BOGUSVALUE"; } // To keep it secure but for looks, we don't return a real value for the password. 
         set { _task.Password = value; }// But, we simulate one so it doesn't look like the password was lost.
      }

      // Public Properties
      [
      Category("Encryption Properties"),
      Description("Whether it is OK to overwrite the destination file if it exists.")      
      ]
      public Boolean OverwriteDestination
      {
         get { return _overwriteDestination; }  
         set { _overwriteDestination = value; }
      }

      [
      Category("Encryption Properties"),
      Description("Select whether you want to encrypt or decrypt the file"),
      TypeConverter(typeof(SSIS2K8.Samples.Tasks.CryptoTask.CryptoTask.EncryptOp))
      ]
      public SSIS2K8.Samples.Tasks.CryptoTask.CryptoTask.EncryptOp EncryptionOption
      {
         get { return _EncryptOp; }
         set { _EncryptOp = value; }
      }

      [
         Category("Connections"),
         Description("The source connection"),
         TypeConverter(typeof(FileConnections))
      ]
      public string SourceConnection
      {
         get { return _sourceConnection; }
         set { _sourceConnection = value; }
      }

      [         
         Category("Connections"),
         Description("The destination connection"),
         TypeConverter(typeof(FileConnections))
      ]
      public string DestinationConnection
      {
         get { return _destConnection; }
         set { _destConnection = value; }
      }
      #endregion Properties

   }
   #endregion SettingsNode
}
